# SPDX-License-Identifier: Apache-2.0
# Copyright 2023 John Mille <john@ews-network.net>

from __future__ import annotations

import json

from cdk_proxy_api_client.interceptors import Interceptors
from cdk_proxy_api_client.proxy_api import ProxyClient
from compose_x_common.compose_x_common import keyisset, set_else_none

from cdk_gw_tools.cli_tools import load_config_file


def interceptors_actions(proxy: ProxyClient, action: str, **kwargs):
    """Triggers function according to CLI Input"""

    _interceptor_client = Interceptors(proxy)
    if action == "list":
        _interceptors = _interceptor_client.list_vcluster_interceptors(
            vcluster_name=kwargs["vcluster_name"], as_list=True
        )
        if keyisset("IgnoreReadOnly", kwargs):
            return [
                _interceptor
                for _interceptor in _interceptors.values()
                if _interceptor["pluginClass"]
                != "io.conduktor.gateway.interceptor.safeguard.ReadOnlyTopicPolicyPlugin"
            ]
        return _interceptors
    elif action == "create-update":
        with open(kwargs["config"]) as config_fd:
            config_raw = config_fd.read()
            config_json = json.loads(config_raw)
        _req = _interceptor_client.create_vcluster_interceptor(
            vcluster_name=kwargs["vcluster_name"],
            interceptor_name=config_json["name"],
            plugin_class=config_json["pluginClass"],
            priority=config_json["priority"],
            config=config_json["config"],
            username=set_else_none("vcluster_username", kwargs, None),
        )
        return _req
    elif action == "delete":
        _req = _interceptor_client.delete_vcluster_interceptor(
            kwargs["vcluster_name"],
            kwargs["interceptor_name"],
            set_else_none("vcluster_username", kwargs, None),
        )
        return _req
    elif action == "import-from-config":
        _loaded_config = load_config_file(kwargs["InputConfigFile"])
        if not keyisset("interceptors", _loaded_config):
            return
        from cdk_gw_tools.cli_tools.set_update_interceptors import (
            set_update_interceptors,
        )

        _req = set_update_interceptors(
            proxy, kwargs["vcluster_name"], _loaded_config["interceptors"]
        )
    return None



#   SPDX-License-Identifier: Apache-2.0
#   Copyright 2023 John Mille <john@ews-network.net>

"""Handles retrieving the configuration file, defining clients, and returning them to subsequent CLI calls"""

from __future__ import annotations

import json

import yaml

try:
    from yaml import Loader as Loader
except ImportError:
    from yaml import CLoader as Loader

from json import loads

from boto3.session import Session
from cdk_proxy_api_client.proxy_api import ApiClient
from compose_x_common.compose_x_common import keyisset, set_else_none
from importlib_resources import files as pkg_files
from jsonschema import validate

from cdk_gw_tools.cli_tools import load_config_file

DEFAULT_SCHEMA_PATH = pkg_files("cdk_gw_tools").joinpath(
    "specs/profiles_config.spec.json"
)


def import_clients(
    config_file: str, client: str = None, schema: dict = None
) -> dict[str, ApiClient]:
    """
    Function that will validate input from specification, then create Proxy client based on the configuration defined.
    It returns a mapping with the profile name and the associated client.
    If client is set, only cares about that one client
    """
    content = load_config_file(config_file)
    if not schema:
        schema = loads(DEFAULT_SCHEMA_PATH.read_text())
    validate(content, schema)
    if client and client not in content:
        raise KeyError(f"Profile {client} not found in definition")

    client_profiles: dict = {}
    for profile, profile_config in content.items():
        if client and profile != client:
            continue
        url = set_else_none("Url", profile_config)
        if not url:
            raise KeyError(f"Url not defined for profile {profile}")
        username = set_else_none("Username", profile_config)
        password = set_else_none("Password", profile_config)
        aws_secrets_manager = set_else_none("AWSSecretsManager", profile_config)
        if username and password:
            client_profiles[profile] = ApiClient(
                url=url, username=username, password=password
            )
        elif aws_secrets_manager:
            client_profiles[profile] = set_profile_from_aws_secret(
                profile, url, aws_secrets_manager
            )
    return client_profiles


def set_profile_from_aws_secret(profile: str, url: str, aws_config: dict) -> ApiClient:
    """
    Uses the AWSSecretsManager configuration
    """
    session = (
        Session(profile_name=aws_config["ProfileName"])
        if keyisset("ProfileName", aws_config)
        else Session()
    )
    client = session.client("secretsmanager")
    user = set_else_none("Username", aws_config)
    try:
        secret_r = client.get_secret_value(SecretId=aws_config["SecretId"])
    except Exception as error:
        print(error)
        raise
    secret_format = set_else_none("Format", aws_config, "yaml")
    if secret_format == "yaml":
        secret_content = yaml.load(secret_r["SecretString"], Loader=Loader)
    else:
        secret_content = json.loads(secret_r["SecretString"])

    if not isinstance(secret_content, list):
        raise TypeError("The content of the secret must be a list of dict")

    for user_defined in secret_content:
        if user and user_defined["username"] == user:
            if not keyisset("admin", user_defined):
                print("The user {} is not admin. Some API calls might fail")
            return ApiClient(url=url, username=user, password=user_defined["password"])

        elif not user:
            if keyisset("admin", user_defined):
                # print(
                #     "Using {} for profile {}".format(user_defined["username"], profile)
                # )
                return ApiClient(
                    url=url,
                    username=user_defined["username"],
                    password=user_defined["password"],
                )

    raise LookupError(f"Unable to find an admin user to make API calls to {url}")



# generated by datamodel-codegen:
#   filename:  user-mappings.spec.json
#   timestamp: 2023-12-04T17:06:08+00:00

from __future__ import annotations

from dataclasses import dataclass
from typing import Dict, List, Optional

Identities = List[str]


@dataclass
class UserMappings:
    identities: Identities


@dataclass
class UserMappingsConfig:
    userMappings: Optional[Dict[str, UserMappings]] = None



# SPDX-License-Identifier: Apache-2.0
# Copyright 2023 John Mille <john@ews-network.net>

from __future__ import annotations

from dataclasses import asdict

from cdk_proxy_api_client.proxy_api import ProxyClient
from cdk_proxy_api_client.usermappings import UserMappings
from cdk_proxy_api_client.vclusters import VirturalClusters
from compose_x_common.compose_x_common import keyisset

from cdk_gw_tools.cli import format_return
from cdk_gw_tools.cli_tools import load_config_file
from cdk_gw_tools.cli_tools.user_mappings_tools import (
    create_export_config,
    import_mappings_from_file,
    validate_identities_are_unique,
)


@format_return
def gw_user_mappings_actions(proxy: ProxyClient, action: str, **kwargs) -> dict:
    vclusters = VirturalClusters(proxy)
    user_mappings = UserMappings(proxy)
    if action == "export":
        req = create_export_config(vclusters, user_mappings)
    elif action == "import":
        config = load_config_file(kwargs["import_file"])
        req = import_mappings_from_file(
            vclusters, user_mappings, config, keyisset("remove_unset", kwargs)
        )
    elif action == "validate":
        config = load_config_file(kwargs["import_file"])
        validate_identities_are_unique(config)
    else:
        raise NotImplementedError(f"Action {action} is not implemented yet.")
    return asdict(req)



from __future__ import annotations

from os import path
from typing import Union

from cdk_proxy_api_client.proxy_api import ProxyClient
from cdk_proxy_api_client.vclusters import VirturalClusters
from compose_x_common.compose_x_common import keyisset, set_else_none

from cdk_gw_tools.cli_tools import load_config_file
from cdk_gw_tools.cli_tools.import_tenants_mappings import import_tenants_mappings


def format_vcluster_mappings_list(
    vcluster_name: str, req: list, options: dict
) -> Union[dict, list]:
    """Parses options to manipulate the returned list/dict"""
    mappings_list: list = []
    for _mapping in req:
        if keyisset("no_concentrated", options):
            if not keyisset("concentrated", _mapping):
                mappings_list.append(_mapping)
        elif keyisset("mapped_only", options):
            if not (
                keyisset("concentrated", _mapping)
                or _mapping["physicalTopicName"].startswith(vcluster_name)
            ):
                mappings_list.append(_mapping)
        else:
            mappings_list.append(_mapping)
    if keyisset("as_import_config", options):
        return {"tenant": vcluster_name, "mappings": mappings_list}
    return mappings_list


def tenant_mappings_actions(
    proxy: ProxyClient, vcluster: VirturalClusters, action: str, **kwargs
):
    """Manages actions for mappings vClusters actions"""

    vcluster_name = set_else_none("vcluster_name", kwargs)
    if action == "list":
        req = format_vcluster_mappings_list(
            vcluster_name,
            vcluster.list_vcluster_topic_mappings(vcluster_name).json(),
            kwargs,
        )
    elif action == "import-from-vclusters-config":
        content = load_config_file(path.abspath(kwargs["import_config_file"]))
        req = import_tenants_mappings(proxy, content, vcluster_name)
    elif action == "create":
        req = vcluster.create_vcluster_topic_mapping(
            vcluster=vcluster_name,
            logical_topic_name=kwargs["logical_topic_name"],
            physical_topic_name=kwargs["physical_topic_name"],
            read_only=keyisset("ReadOnly", kwargs),
            concentrated=keyisset("concentrated", kwargs),
            cluster_id=kwargs.get("cluster_id"),
        )
    elif action == "import-from-tenant":
        source_tenant = kwargs.pop("source_tenant")
        content = {
            "vcluster_name": vcluster_name,
            "mappings": [],
            "ignore_duplicates_conflict": True,
            "import_from_tenant": {"include_regex": [rf"^{source_tenant}$"]},
        }
        req = import_tenants_mappings(proxy, content, vcluster_name)
    elif action == "delete-topic-mapping":
        to_delete = kwargs.pop("logicalTopicName")
        req = vcluster.delete_vcluster_topic_mapping(
            vcluster=vcluster_name, logical_topic_name=to_delete
        )
    elif action == "delete-all-mappings":
        req = vcluster.delete_vcluster_topics_mappings(vcluster_name)
    else:
        raise NotImplementedError(f"Action {action} not yet implemented.")
    return req



from __future__ import annotations

from cdk_proxy_api_client.proxy_api import ProxyClient
from cdk_proxy_api_client.usermappings import UserMappings


def user_mappings_actions(proxy: ProxyClient, action: str, **kwargs):
    """
    Handles vCluster OAuth mappings actions
    """
    user_mappings = UserMappings(proxy)
    if action == "list":
        return user_mappings.list_mappings(kwargs["vcluster_name"])
    elif action == "create":
        return user_mappings.create_mapping(kwargs["vcluster_name"], kwargs["username"])
    elif action == "delete":
        return user_mappings.delete_mapping(kwargs["vcluster_name"], kwargs["username"])
    else:
        raise NotImplementedError(
            f"Action {action} not yet implemented for user mappings."
        )



# SPDX-License-Identifier: Apache-2.0
# Copyright 2023 John Mille <john@ews-network.net>

"""Manages the vclusters <action> of the CLI"""

from __future__ import annotations

from cdk_proxy_api_client.proxy_api import ProxyClient
from cdk_proxy_api_client.vclusters import VirturalClusters

from cdk_gw_tools.cli import format_return
from cdk_gw_tools.cli.actions.vclusters.auth import auth_actions
from cdk_gw_tools.cli.actions.vclusters.interceptors import interceptors_actions
from cdk_gw_tools.cli.actions.vclusters.topic_mappings import tenant_mappings_actions
from cdk_gw_tools.cli.actions.vclusters.user_mappings import user_mappings_actions


@format_return
def vclusters_actions(proxy: ProxyClient, action: str, **kwargs):
    """Manages execution of vClusters"""
    vclusters = VirturalClusters(proxy)
    if action == "list":
        req = vclusters.list_vclusters(as_list=True)
    elif action == "auth":
        req = auth_actions(vclusters, kwargs.pop("sub_action"), **kwargs)
    elif action == "mappings":
        req = tenant_mappings_actions(
            proxy, vclusters, kwargs.pop("sub_action"), **kwargs
        )
    elif action == "interceptors":
        req = interceptors_actions(proxy, kwargs.pop("sub_action"), **kwargs)
    elif action == "user-mappings":
        req = user_mappings_actions(proxy, kwargs.pop("sub_action"), **kwargs)
    else:
        raise NotImplementedError(f"Action {action} not yet implemented.")
    return req



# SPDX-License-Identifier: Apache-2.0
# Copyright 2023 John Mille <john@ews-network.net>

from __future__ import annotations

from argparse import ArgumentParser

VCLUSTER_PARSER = ArgumentParser(add_help=False)
VCLUSTER_PARSER.add_argument(
    "--vcluster-name",
    dest="vcluster_name",
    required=True,
    help="Name of the vcluster to make operations for",
)


def set_gw_user_mappings_actions_parsers(gw_user_mappings_subparsers):
    import_parser = gw_user_mappings_subparsers.add_parser(
        name="import", help="Import user-mappings from config-file"
    )
    import_parser.add_argument(
        "-f",
        "--config-file",
        dest="import_file",
        type=str,
        help="Path to the import config file",
    )
    import_parser.add_argument(
        "--remove-unset",
        action="store_true",
        help="If true, remove mappings on GW not defined in the config file",
    )

    export_parser = gw_user_mappings_subparsers.add_parser(
        name="export", help="Export user-mappings from config-file"
    )
    export_parser.add_argument(
        "-f",
        "--config-file",
        dest="import_file",
        type=str,
        help="Path to the import config file",
        required=False,
    )



# SPDX-License-Identifier: Apache-2.0
# Copyright 2023 John Mille <john@ews-network.net>

from __future__ import annotations

from argparse import ArgumentParser
from os import environ

from cdk_gw_tools.cli.gw_parsers.plugins import set_plugings_actions_parsers
from cdk_gw_tools.cli.gw_parsers.user_mappings import (
    set_gw_user_mappings_actions_parsers,
)
from cdk_gw_tools.cli.vclusters_parser import set_vclusters_actions_parsers


def set_parser():
    main_parser = ArgumentParser("CDK Proxy CLI", add_help=True)
    main_parser.add_argument(
        "--format",
        "--output-format",
        dest="output_format",
        help="output format",
        default="yaml",
    )
    main_parser.add_argument(
        "--log-level", dest="loglevel", type=str, help="Set loglevel", required=False
    )
    main_parser.add_argument("--url", required=False)
    main_parser.add_argument("--username", required=False)
    main_parser.add_argument("--password", required=False)
    main_parser.add_argument(
        "-c",
        "--config-file",
        type=str,
        help="Path to the profiles files",
        default="{}/.cdk_gw.yaml".format(environ.get("HOME", ".")),
    )
    main_parser.add_argument(
        "-p",
        "--profile-name",
        type=str,
        help="Name of the profile to use to make API Calls",
    )

    cmd_parser = main_parser.add_subparsers(dest="category", help="Resources to manage")
    vclusters_parser = cmd_parser.add_parser(
        name="vclusters",
        help="Manages vClusters",
    )
    vclusters_subparsers = vclusters_parser.add_subparsers(
        dest="action", help="vCluster management"
    )
    set_vclusters_actions_parsers(vclusters_subparsers)

    plugins_parser = cmd_parser.add_parser(name="plugins", help="Manage plugins")
    plugins_subparsers = plugins_parser.add_subparsers(
        dest="action", help="Manage plugins actions"
    )
    set_plugings_actions_parsers(plugins_subparsers)

    gw_user_mappings_parser = cmd_parser.add_parser(
        name="user-mappings", help="Manage user mappings at the GW level"
    )
    gw_user_mappings_subparsers = gw_user_mappings_parser.add_subparsers(
        dest="action", help="Manage GW user mappings actions"
    )
    set_gw_user_mappings_actions_parsers(gw_user_mappings_subparsers)

    return main_parser



# SPDX-License-Identifier: Apache-2.0
# Copyright 2023 John Mille <john@ews-network.net>

from __future__ import annotations

from cdk_proxy_api_client.vclusters import VirturalClusters
from compose_x_common.compose_x_common import keyisset


def auth_actions(vcluster: VirturalClusters, action: str, **kwargs):
    """Manages actions for auth vClusters subparser"""
    username = kwargs.get("username") or kwargs["vcluster_name"]
    if action == "create":
        req = vcluster.create_vcluster_user_token(
            vcluster=kwargs.get("vcluster_name"),
            username=username,
            lifetime_in_seconds=int(kwargs.get("token_lifetime_in_seconds")),
            token_only=keyisset("token_only", kwargs),
        )
    else:
        raise NotImplementedError(f"Action {action} not yet implemented.")
    if keyisset("as_kafka_config", kwargs):
        req = req.json()
        return """security.protocol=SASL_SSL
sasl.mechanism=PLAIN
sasl.jaas.config=org.apache.kafka.common.security.plain.PlainLoginModule required username="{}" password="{}";
client.id=CLI_{}""".format(
            username, req["token"], username
        )
    return req



#   SPDX-License-Identifier: Apache-2.0
#   Copyright 2023 John Mille <john@ews-network.net>

"""Functions to help with the user-mappings management"""

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from cdk_proxy_api_client.usermappings import UserMappings
    from cdk_proxy_api_client.vclusters import VirturalClusters

from dacite import from_dict

from cdk_gw_tools.specs.user_mappings import UserMappingsConfig


def create_export_config(
    vclusters: VirturalClusters, user_mappings: UserMappings
) -> UserMappingsConfig:
    """Lists all vClusters and returns the"""
    user_mappings_config: dict = {}
    mappings_config: dict = {"userMappings": user_mappings_config}
    _vclusters_list = vclusters.list_vclusters(as_list=True)
    for _vcluster_name in _vclusters_list["vclusters"]:
        _vcluster_mappings = user_mappings.list_mappings(vcluster=_vcluster_name).json()
        if _vcluster_mappings:
            user_mappings_config[_vcluster_name] = {"identities": _vcluster_mappings}
    return from_dict(UserMappingsConfig, mappings_config)


def validate_identities_are_unique(config: dict) -> UserMappingsConfig:
    mappings_config = from_dict(UserMappingsConfig, config)
    identities_recorded: list = []
    for definition in mappings_config.userMappings.values():
        for identity in definition.identities:
            if identity in identities_recorded:
                raise ValueError(f"Identity {identity} is defined multiple times.")
            identities_recorded.append(identity)
    return mappings_config


def import_mappings_from_file(
    vclusters, user_mappings, config: dict, remove_unset: bool = False
) -> UserMappingsConfig:
    mappings_config = validate_identities_are_unique(config)

    for vcluster_name, definition in mappings_config.userMappings.items():
        _existing_mappings = user_mappings.list_mappings(vcluster_name).json()
        for identity in definition.identities:
            if identity not in _existing_mappings:
                user_mappings.create_mapping(vcluster_name, identity)
        if remove_unset:
            for identity in _existing_mappings:
                if identity not in definition.identities:
                    print(
                        f"{vcluster_name} - {identity}:"
                        " identity is not defined in configuration, but present in existing mappings. Deleting."
                    )
                    user_mappings.delete_mapping(vcluster_name, identity)
    return create_export_config(vclusters, user_mappings)



# SPDX-License-Identifier: Apache-2.0
# Copyright 2023 John Mille <john@ews-network.net>

from __future__ import annotations

from cdk_proxy_api_client.plugins import Plugins
from cdk_proxy_api_client.proxy_api import ProxyClient
from compose_x_common.compose_x_common import keyisset

from cdk_gw_tools.cli import format_return


@format_return
def plugins_actions(proxy: ProxyClient, action: str, **kwargs):
    _plugins = Plugins(proxy)
    if action == "list":
        req = _plugins.list_all_plugins(
            extended=keyisset("extended", kwargs), as_list=keyisset("as_list", kwargs)
        )
    else:
        raise NotImplementedError(f"Action {action} is not implemented yet.")
    return req
